package com.codigoparallevar.minicards.parts.buttons;

import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.util.Log;

import com.codigoparallevar.minicards.types.InputConnector;
import com.codigoparallevar.minicards.types.Moveable;
import com.codigoparallevar.minicards.types.OutputConnector;
import com.codigoparallevar.minicards.types.Part;
import com.codigoparallevar.minicards.types.PartGrid;
import com.codigoparallevar.minicards.types.RoundInputConnector;
import com.codigoparallevar.minicards.types.RoundOutputConnector;

import org.json.JSONException;
import org.json.JSONObject;

import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

public class RoundButton implements Part {

    private final PartGrid _partGrid;
    private int _xCenter;
    private int _yCenter;
    private final int _innerRadius;
    private final int _outerRadius;
    private final int _outerRadiusThickness = 10;
    private final int _pathRunWay = 200;
    private List<OutputConnector> _outputConnectors;
    private final RoundOutputConnector _pressedOuputConnector;

    public RoundButton(PartGrid partGrid, int xCenter, int yCenter, int innerRadius, int outerRadius) {
        _partGrid = partGrid;
        _xCenter = xCenter;
        _yCenter = yCenter;
        _innerRadius = innerRadius;
        _outerRadius = outerRadius;

        // Create connectors
        _pressedOuputConnector = new RoundOutputConnector(
                this,
                _partGrid,
                getOutputConnectorCenterX(), getOutputConnectorCenterY(),
                getOutputConnectRadius());

        _outputConnectors = new LinkedList<>();
        _outputConnectors.add(_pressedOuputConnector);

    }

    @Override
    public int getLeft() {
        return _xCenter - _outerRadius / 2;
    }

    @Override
    public int getRight() {
        return _xCenter + _outerRadius / 2;
    }

    @Override
    public int getTop() {
        return _yCenter - _outerRadius / 2;
    }

    @Override
    public int getBottom() {
        return _yCenter + _outerRadius / 2;
    }

    @Override
    public void draw(Canvas canvas, boolean devMode) {
        Paint foregroundPaint = new Paint(Paint.ANTI_ALIAS_FLAG);
        foregroundPaint.setColor(Color.WHITE);

        Paint backgroundPaint = new Paint(Paint.ANTI_ALIAS_FLAG);
        backgroundPaint.setColor(Color.BLACK);

        if (devMode){
            drawConnector(canvas);
            drawWires(canvas, devMode);
        }

        canvas.drawCircle(_xCenter, _yCenter, _outerRadius, foregroundPaint);
        canvas.drawCircle(_xCenter, _yCenter, _outerRadius - _outerRadiusThickness, backgroundPaint);
        canvas.drawCircle(_xCenter, _yCenter, _innerRadius, foregroundPaint);
    }

    private void drawConnector(Canvas canvas) {
        Paint connectorPaint = new Paint(Paint.ANTI_ALIAS_FLAG);
        connectorPaint.setColor(Color.RED);

        canvas.drawCircle(getOutputConnectorCenterX(), getOutputConnectorCenterY(),
                getOutputConnectRadius(),
                connectorPaint);
    }

    private int getOutputConnectorCenterX() {
        return _xCenter + _outerRadius;
    }

    private int getOutputConnectorCenterY() {
        return _yCenter;
    }

    private int getOutputConnectRadius() {
        return _innerRadius;
    }

    private void drawWires(Canvas canvas, boolean devMode) {
        for (OutputConnector outputConnector : _outputConnectors){
            outputConnector.drawWires(canvas, devMode);
        }
    }

    @Override
    public void moveEnd(int x, int y) {
        _xCenter = x;
        _yCenter = y;

        // Move connectors too
        _pressedOuputConnector.updatePosition(
                getOutputConnectorCenterX(),
                getOutputConnectorCenterY());
    }

    @Override
    public void drop(int x, int y) {
        moveEnd(x, y);
    }

    @Override
    public void touched() {
        Log.d("RoundButton", "Round button touched");

        _pressedOuputConnector.sendSignal();
    }

    @Override
    public List<InputConnector> getInputConnectors() {
        return Collections.emptyList();
    }

    @Override
    public List<OutputConnector> getOutputConnectors() {
        return _outputConnectors;
    }

    @Override
    public JSONObject serialize() throws JSONException {
        JSONObject serialized = new JSONObject();

        serialized.put("x_center", _xCenter);
        serialized.put("y_center", _yCenter);
        serialized.put("inner_radius", _innerRadius);
        serialized.put("outer_radius", _outerRadius);
        serialized.put("outer_radius_thickness", _outerRadiusThickness);

        return serialized;
    }

    @Override
    public void sendSignal(RoundInputConnector roundInputConnector) {
        // @TODO: REMOVE THE NEED FOR THIS
    }

    public static Part deserialize(PartGrid partGrid, JSONObject data) throws JSONException {
        int xCenter = data.getInt("x_center");
        int yCenter = data.getInt("y_center");

        int innerRadius = data.getInt("inner_radius");
        int outerRadius = data.getInt("outer_radius");

        return new RoundButton(partGrid, xCenter, yCenter, innerRadius, outerRadius);
    }

    @Override
    public boolean containsPoint(int x, int y) {
        return ((Math.abs(x - _xCenter) <= _outerRadius)
                && (Math.abs(y - _yCenter) <= _outerRadius));
    }

    @Override
    public Moveable getMoveable() {
        return this;
    }
}
